"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const execa = require("execa");
const fs_1 = require("fs");
const os_1 = require("os");
const path_1 = require("path");
const which = require("which");
/**
 * Utility for creating ISO 9660 images.
 *
 * @class Mkiso
 */
class Mkiso extends events_1.EventEmitter {
    /**
     * Creates an instance of Mkiso.
     *
     * @param {string} sourceDir
     * @memberof Mkiso
     */
    constructor(sourceDir) {
        super();
        this.defaultAdapters = {
            darwin: 'hdiutil',
            linux: 'mkisofs',
        };
        this._platform = os_1.platform();
        this._adapter = null;
        this._label = '';
        this._publisher = '';
        this._verbose = false;
        if (!sourceDir || sourceDir.length === 0) {
            throw new Error('Missing required parameter sourceDir');
        }
        this._source = sourceDir;
        this._output = (sourceDir.endsWith(path_1.sep) ? sourceDir.slice(0, -1) : sourceDir).concat('.iso');
    }
    /**
     * Set the volume name.
     *
     * @param {string} name
     * @returns
     * @memberof Mkiso
     */
    label(name) {
        if (typeof name === 'string' && name.length > 0) {
            this._label = name;
        }
        return this;
    }
    /**
     * Set the publisher name.
     *
     * @param {string} name
     * @returns
     * @memberof Mkiso
     */
    publisher(name) {
        if (typeof name === 'string' && name.length > 0) {
            this._publisher = name;
        }
        return this;
    }
    /**
     * Enable the verbose mode.
     *
     * @returns
     * @memberof Mkiso
     */
    verbose() {
        this._verbose = true;
        return this;
    }
    /**
     * Set the iso output file path.
     *
     * @param {string} target
     * @returns
     * @memberof Mkiso
     */
    output(target) {
        if (typeof target === 'string' && target.length > 0) {
            this._output = path_1.extname(target) === '.iso' ? target : target.concat('.iso');
        }
        return this;
    }
    /**
     * Override the default adapter with a custom one.
     * You can use an MkisoAdapter adapter instance
     * or the name of an adapter into the adapters folder or link an external one.
     *
     * @param {(string | Adapter)} adapter
     * @memberof Mkiso
     */
    async adapter(adapter) {
        try {
            if (typeof adapter !== 'string') {
                this._adapter = adapter;
            }
            else {
                const isPath = adapter !== path_1.basename(adapter);
                const m = await Promise.resolve().then(() => require(isPath ? adapter : path_1.resolve(__dirname, 'adapters', adapter)));
                this._adapter = m.default || m;
            }
            return this;
        }
        catch (err) {
            throw err;
        }
    }
    /**
     * Starts the burning process.
     *
     * @param {ReadonlyArray<string>} [args=[]]
     * @returns
     * @memberof Mkiso
     */
    async exec(args = [], options) {
        try {
            const exists = await this.pathExists(this._source);
            if (!exists)
                throw new Error(`${this._source} does not exists`);
            const { command, args: defaults } = await this.loadAdapter();
            const exec = execa(command, [...defaults, ...args, this._source], options);
            // Add event listeners
            this.eventNames().forEach(evt => {
                switch (evt) {
                    case 'progress':
                        exec.stdout.on('data', data => this.emit('progress', data));
                        break;
                    case 'stderr':
                        exec.stderr.on('data', data => this.emit('stderr', data));
                        break;
                    default:
                        exec.on(evt, (...eargs) => this.emit(evt, ...eargs));
                        break;
                }
            });
            return exec;
        }
        catch (err) {
            throw err;
        }
    }
    /**
     * Load the configured adapter.
     *
     * @private
     * @returns
     * @memberof Mkiso
     */
    async loadAdapter() {
        try {
            if (!this._adapter)
                await this.defaultAdapter();
            if (typeof this._adapter !== 'function') {
                throw new Error('The adapter is not valid');
            }
            const adapter = this._adapter();
            if (adapter.command === undefined || adapter.args === undefined) {
                throw new Error('The adapter is malformed');
            }
            // Check if the command exists
            which.sync(adapter.command);
            if (this._label.length > 0)
                adapter.label(this._label);
            if (this._publisher.length > 0)
                adapter.publisher(this._publisher);
            if (this._verbose)
                adapter.verbose();
            adapter.output(this._output);
            return adapter;
        }
        catch (err) {
            throw err;
        }
    }
    /**
     * Set the default adapter based on OS platform.
     *
     * @private
     * @memberof Mkiso
     */
    async defaultAdapter() {
        const adapter = this.defaultAdapters[this._platform];
        if (adapter === undefined) {
            throw new Error(`Failed to find a valid adapter for this platform "${this._platform}"`);
        }
        await this.adapter(adapter);
    }
    /**
     * Promisify access function.
     *
     * @private
     * @param {string} sourcePath
     * @returns
     * @memberof Mkiso
     */
    async pathExists(sourcePath) {
        return new Promise(res => {
            fs_1.access(sourcePath, err => res(!err));
        });
    }
}
function mkiso(sourceDir) {
    return new Mkiso(sourceDir);
}
exports.default = mkiso;
